package com.SocialApp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Random;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.World;
import com.badlogic.gdx.physics.box2d.BodyDef.BodyType;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.List;


public class BuildingGenerator {
	
	public int type; //classify the type of building
	public GameWorld game;
	private int wSize = 64; //wall size

	public BuildingGenerator(int t, GameWorld g) {
		
		type = t;
		game = g;
		
		createBasicStructure();
		
	}
	
	
	protected void createBasicStructure() {
		
		//generate a random basic structure
		generateStructure();
		
	}
	
	
	private void generateStructure() {
		
		int corners, windows, doors; //number of parts for structure
		GameObject cnrParts[];
		
		//determine number of structure components
		Random counter = new Random();
		
		int gen; //generated number
		Vector2 cornerLocs[];
		
		corners = ((gen = (counter.nextInt(8)+5)) % 2 == 0 ? gen : gen-1);  //a minimum of 4 corners required and must be even

		cornerLocs = getLocation(4); //generate corner locations
		
		cnrParts = createGenParts(cornerLocs, "CornerBlock"); //generate corner parts
		
		createGenParts(getLocation(cnrParts, "WallH"), "WallH"); //generate horizontal wall parts based on corner parts
		
		createGenParts(getLocation(cnrParts, "WallV"), "WallV"); //generate vertical wall parts based on corner parts
		
		doors = counter.nextInt(3)+1;
		
		
	}
	
	private GameObject[] createGenParts(Vector2[] partCoords, String objT) {
		
		GameObject parts[] = new GameObject[partCoords.length]; 
		
		for(int i = 0; i < partCoords.length; i++) {
			ObjectGenerator part = new ObjectGenerator(partCoords[i], new Vector2(0, 0), game, objT); //create building object (position, vel, world)
			parts[i] = (GameObject) part.getObjB().getUserData(); //retrieve part
		}
		
		return parts; 

	}
	
	private Vector2[] getLocation(int amount) {
		
		Vector2[] parts = new Vector2[4];

			/*
			Random rand = new Random(); int cPillar;
			//create pillar corners
			parts[0] = new Vector2(-300, 300); //create initial corner			
			parts[(cPillar = rand.nextInt(amount))+cPillar] = new Vector2(300, 150);
			parts[(cPillar = rand.nextInt(amount))+cPillar] = new Vector2(300, -300);
			parts[(cPillar = rand.nextInt(amount))+cPillar] = new Vector2(-300, -150);
			
			for(int ct = 0; ct < amount; ct++) {
				if
				parts[ct] = 
				
			}
			*/
		//Following is sample construction of corners. It will be more random
			parts[0] = new Vector2(-Gdx.graphics.getWidth()/5-Gdx.graphics.getWidth()/8, Gdx.graphics.getHeight()/4); //create corner coordinates relative to screen size	
			parts[1] = new Vector2(Gdx.graphics.getWidth()/3+Gdx.graphics.getWidth()/8, Gdx.graphics.getHeight()/4);
			parts[2] = new Vector2(Gdx.graphics.getWidth()/5+Gdx.graphics.getWidth()/8, -Gdx.graphics.getHeight()/4);
			parts[3] = new Vector2(-Gdx.graphics.getWidth()/3-Gdx.graphics.getWidth()/8, -Gdx.graphics.getHeight()/4);
			
			
		return parts ;
		
	}
	
	private Vector2[] getLocation(GameObject[] depPart, String type) { //retrieve location vectors for all parts of the structure type
	
		ArrayList<Vector2> parts = new ArrayList<Vector2>(); //create new array list for storing part locations
		
		if(type.contains("Wall")) { //if type is wall
			
			for(int cnrCt = 1; cnrCt <= depPart.length; cnrCt++) {  //for every corner
				
				int sCnr, eCnr; //start corner, end corner
				
				//ensure corner index stay within range
				sCnr = (cnrCt != depPart.length ? cnrCt : cnrCt-1); 
				eCnr = (cnrCt != depPart.length ? cnrCt-1 : 0);
				
				float dst = depPart[sCnr].origV.dst(depPart[eCnr].origV) - wSize; //get distance between corners
				
				if(type.contains("WallH") && cnrCt%2 != 0) { //if corners are horizontal to each other				
					parts.addAll(createHWall(depPart, sCnr, eCnr, dst));
				}
				else if (type.contains("WallV") && cnrCt%2 == 0) { //if corners are vertical to each other
					parts.addAll(createVWall(depPart, sCnr, eCnr, dst));
				}
			}
		}
		
		return convertListToArray(parts);
		
	}
	
	private ArrayList<Vector2> createVWall(GameObject[] depPart, int sCnr, int eCnr, float dst) {
		
		ArrayList<Vector2> parts = new ArrayList<Vector2>(); //create new array list for storing part locations
		
		Vector2[] arg = {depPart[eCnr].origV, depPart[sCnr].origV};
		float incD = getXPointDist(arg); float tIncD = 0; //determine the incremental distance between both corners
		float orig = Math.min(depPart[eCnr].origV.y+wSize, depPart[sCnr].origV.y+wSize); //get lowest starting yCoord
		for(float ptY = orig; ptY < dst+orig-wSize; ptY+=wSize) {//for every wall part after the corner
			tIncD+=incD; // increment the x coordinate for diagonal placement
			parts.add(new Vector2(depPart[sCnr].origV.x+tIncD, ptY)); //add coordinates for part
		}
		
		return parts;
		
	}
	
	private ArrayList<Vector2> createHWall(GameObject[] depPart, int sCnr, int eCnr, float dst) {
		
		ArrayList<Vector2> parts = new ArrayList<Vector2>(); //create new array list for storing part locations
		
		float orig = Math.min(depPart[eCnr].origV.x, depPart[sCnr].origV.x)+wSize; //get lowest starting xCoord
		for(float ptX = orig; ptX < dst+orig; ptX+=wSize) { //for every wall part after the corner
			parts.add(new Vector2(ptX, depPart[sCnr].origV.y)); //add coordinates for part
		}
			
		return parts;
		
	}
	
	private Vector2[] convertListToArray(ArrayList<Vector2> l) {
		
		Vector2[] array; ListIterator<Vector2> it;
		array = new Vector2[l.size()]; int ct = 0;
		
		while((it = l.listIterator()).hasNext()) {
			array[ct++] = (Vector2) it.next();
			it.remove();
		}
			
		return array;
		
	}
	
	private float getXPointDist(Vector2[] points) {
		
		float s1 = Math.abs(points[0].y - points[1].y); //get horizontal length between points
		float partAmt  = s1/wSize; //determine amount of parts between points
		float xDist = Math.abs(points[0].x - points[1].x)/partAmt; //determine the length of the parts
		
		
		return xDist;
		
	}
	

}
