package com.SocialApp;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.BodyDef.BodyType;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Scaling;

public class ObjectGenerator {
	
	private Vector2 objCoords, objVel = null; //properties for Box2d object
	private GameWorld game; // Box2d world
	private Body objB; //Box2d body
	private String objN; //Name of object
	private short group[] = 	{0,1};
	private short category[] = 	{2,1};
	private short mask[] = 		{9,2};
	private int bodySize = 64;
	

	public ObjectGenerator(Vector2 bodyCoords, Vector2 bodyVel, GameWorld world, String objName) {
		
		objCoords = bodyCoords;
		game = world;
		objVel = bodyVel;
		objN = objName;
		
		// resize body if required
		if(objN.contains("Char"))
			bodySize = 32;
		
		createObjBody();
		createObjActor(objName);
		
	}
	
	private void createObjBody() {
		
		//create fixture / body definitions
		FixtureDef fd = new FixtureDef();
		BodyDef bd = new BodyDef();
			
		PolygonShape partShape = new PolygonShape(); //create shape for body
		partShape.setAsBox((float)(bodySize*0.5), (float)(bodySize*0.5), new Vector2(objCoords), 0); //place shape at body coordinates
		
		fd.shape = partShape; //assign shape to body fixture
		
		if(objN.contains("Char")) { // if body is a game character
			
			createCharCollision(bd, fd);
		}
		else { //if body is game obstacle
			
			createObstacleCollision(bd, fd);
			
		}
		
		//create body
		objB = game.world.createBody(bd);
		objB.createFixture(fd);
		
		//create body Properties
		if(objN.contains("Char"))
			objB.setUserData(new CharacterObject(objB, new Vector2(objCoords)));
		else
			objB.setUserData(new GameObject(objB, new Vector2(objCoords)));	
		
	}
	
	private void createCharCollision(BodyDef bd, FixtureDef fd) {
		
		//create body using dynamic physics
		bd.type = BodyType.DynamicBody;
		
	    //create character collision class (1)
	    fd.filter.groupIndex = group[1];
	    fd.filter.categoryBits =  category[1];
	    fd.filter.maskBits = mask[1];
	    
	    //give body elasticity
	    fd.restitution = (float) 1.0;

		
	}
	
	private void createObstacleCollision(BodyDef bd, FixtureDef fd) {
		
		//create immovable body
		bd.type = BodyType.StaticBody;
		
		//create static collision class (class 0)
	    fd.filter.groupIndex = group[0];
	    fd.filter.categoryBits =  category[0];
	    fd.filter.maskBits = mask[0];
		
	}
	
	private void createObjActor(String objType) {
		
		//create actor for object
		Actor objActor = new Image(new Texture(Gdx.files.internal(objType+".png")), Scaling.none, 0, objType);

		game.stage.addActor(objActor);
		
		//set actor to body properties
		GameObject bM = (GameObject) objB.getUserData();
		bM.setActor(objActor);
		
		
	}
	
	public Body getObjB() {
		return objB;
	}

}
