package com.SocialApp;

import java.util.Iterator;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.World;
import com.badlogic.gdx.scenes.scene2d.Stage;

public class SetUpWorld {
	
	public World world; //Global world instance for physics
	public Stage stage; //Global state instance for acting
	public Collision collision;
	
	public SetUpWorld() {
		
		createWorld();
	}
	
	private void createWorld() {
		
		world = new World(new Vector2(0,0), true); //create a new world for object physics
		stage = new Stage(Gdx.graphics.getWidth(), Gdx.graphics.getHeight(), true); //create stage for object acting and rendering
		collision = new Collision(); //create
		
		//create game buildings
		createLevelBuilding(0);
		
		//create game characters
		createLevelNPC(0);
	}
	
	private void buildLevel() {
		
		createLevelBuilding(1); //create building for particular level
		
	}
	
	private void createLevelBuilding(int type) {
		
		new BuildingGenerator(type, this); //invoke building generator according to building type
		
		//characters need to have "char" within the string parameter
//		new ObjectGenerator(new Vector2(0,0), new Vector2(0,100), this, "testNPChar");
//		new ObjectGenerator(new Vector2(50,50), new Vector2(-100,0), this, "testNPChar");
//		new ObjectGenerator(new Vector2(-50,-10), new Vector2(100,0), this, "testNPChar");
//		new ObjectGenerator(new Vector2(20,-50), new Vector2(100,-100), this, "testNPChar");
//		new ObjectGenerator(new Vector2(0,-50), new Vector2(100,0), this, "testNPChar");
//		new ObjectGenerator(new Vector2(10,10), new Vector2(100,150), this, "testNPChar");
		
	}
	
	private void createLevelNPC(int type) {
		
		//characters need to have "char" within the string parameter
		new ObjectGenerator(new Vector2(0,0), new Vector2(0,100), this, "testNPChar");
		new ObjectGenerator(new Vector2(50,50), new Vector2(-100,0), this, "testNPChar");
		new ObjectGenerator(new Vector2(-50,-10), new Vector2(100,0), this, "testNPChar");

	}
	
	public void moveObjects() {
		
		Body b;
		Iterator<Body> it = world.getBodies();
		
		//cycle through all available bodies
		for(int bCt = 0; bCt < world.getBodyCount(); bCt++) {
			b = it.next();
			BodyMovement bM = (BodyMovement) b.getUserData(); //access body properties
			bM.updateTexture(); //update texture (actor) coordinates
		}

	}

}
