package com.SocialApp;

import com.badlogic.gdx.ApplicationListener;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;

public class SocialApp implements ApplicationListener {
	private GameWorld gameWorld;
	
	//for debugging purposes
	private Box2DDebugRenderer bodyRender;
	private OrthographicCamera camera;
	
	@Override
	public void create() {		
		gameWorld = new GameWorld(); //create the game world
		
		//for debugging purposes
		bodyRender = new Box2DDebugRenderer(true, false, false, false);
		camera = new OrthographicCamera();
		camera.setToOrtho(false, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
	}

	@Override
	public void dispose() {
		gameWorld.stage.dispose();
		gameWorld.world.dispose();
	}

	@Override
	public void render() {		
		Gdx.gl.glClearColor(1, 1, 1, 1);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
		
		//handle stage acting
		gameWorld.stage.act(1);
		gameWorld.stage.draw();
		
		//handle physics updating
		gameWorld.world.step((float) (1), 6, 4); //short time step required for android devices(slow processing)
		bodyRender.render(gameWorld.world, camera.projection); //debugging
		
		//handle game events
		gameWorld.updateGame();
	}

	@Override
	public void resize(int width, int height) {
	}

	@Override
	public void pause() {
	}

	@Override
	public void resume() {
	}
}
